@@ -137,7 +137,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_initNativeLibrary(void)
   return TRUE;
 }
 
-static guint
+guint
 jni_main_idle_add(GSourceFunc    function, gpointer       data)
 {
   JAW_DEBUG_C("%p, %p", function, data);
@@ -231,20 +231,24 @@ typedef struct _CallbackPara {
 } CallbackPara;
 
 static CallbackPara*
-alloc_callback_para (JNIEnv *jniEnv, jobject ac)
+alloc_callback_para (JNIEnv *jniEnv, jobject jAccContext)
 {
-  JAW_DEBUG_C("%p, %p", jniEnv, ac);
-  if (ac == NULL)
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+
+  JAW_DEBUG_C("%p, %p", jniEnv, global_ac);
+  if (global_ac == NULL)
     return NULL;
-  JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, ac);
+
+  JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
     JAW_DEBUG_I("jaw_impl == NULL");
     return NULL;
   }
+
   g_object_ref(G_OBJECT(jaw_impl));
   CallbackPara *para = g_new(CallbackPara, 1);
-  para->global_ac = ac;
+  para->global_ac = global_ac;
   para->jaw_impl = jaw_impl;
   para->child_impl = NULL;
   para->args = NULL;
@@ -273,6 +277,9 @@ free_callback_para (CallbackPara *para)
 
   g_object_unref(G_OBJECT(para->jaw_impl));
 
+  if (para->child_impl)
+    g_object_unref(G_OBJECT(para->child_impl));
+
   if (para->args) {
     (*jniEnv)->DeleteGlobalRef(jniEnv, para->args);
   }
@@ -338,9 +345,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_focusNotify(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(focus_notify_handler, para);
 }
 
@@ -395,9 +401,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowOpen(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   para->is_toplevel = jIsToplevel;
   jni_main_idle_add(window_open_handler, para);
 }
@@ -451,9 +456,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowClose(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   para->is_toplevel = jIsToplevel;
   jni_main_idle_add(window_close_handler, para);
 }
@@ -482,9 +486,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMinimize(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(window_minimize_handler, para);
 }
 
@@ -511,9 +514,8 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMaximize(JN
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac );
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext );
   jni_main_idle_add(window_maximize_handler, para);
 }
 
@@ -540,9 +542,8 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowRestore(JNI
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(window_restore_handler, para);
 }
 
@@ -569,9 +570,8 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowActivate(JN
       JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(window_activate_handler, para);
 }
 
@@ -599,9 +599,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowDeactivate(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(window_deactivate_handler, para);
 }
 
@@ -629,9 +628,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowStateChange(JNIEnv *jniEnv
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(window_state_change_handler, para);
 }
 
@@ -713,27 +711,17 @@ signal_emit_handler (gpointer p)
                             "children_changed::add",
                             child_index,
                             para->child_impl);
-      if (G_OBJECT(atk_obj) != NULL)
-        g_object_ref(G_OBJECT(atk_obj));
       break;
     }
     case Sig_Object_Children_Changed_Remove:
     {
       gint child_index = get_int_value(jniEnv,
                                        (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
-      jobject child_ac = (*jniEnv)->GetObjectArrayElement(jniEnv, args, 1);
-      JawImpl *child_impl = jaw_impl_find_instance(jniEnv, child_ac);
-      if (!child_impl)
-      {
-        break;
-      }
 
       g_signal_emit_by_name(atk_obj,
                             "children_changed::remove",
                             child_index,
-                            child_impl);
-      if (G_OBJECT(atk_obj) != NULL)
-        g_object_unref(G_OBJECT(atk_obj));
+                            para->child_impl);
       break;
     }
     case Sig_Object_Active_Descendant_Changed:
@@ -930,10 +918,9 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
     return;
   }
 
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
   jobjectArray global_args = (jobjectArray)(*jniEnv)->NewGlobalRef(jniEnv, args);
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   para->ac = jAccContext;
   para->signal_id = (gint)id;
   para->args = global_args;
@@ -943,7 +930,6 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
     case Sig_Text_Property_Changed_Insert:
     case Sig_Text_Property_Changed_Delete:
     case Sig_Text_Property_Changed_Replace:
-    case Sig_Object_Children_Changed_Remove:
     case Sig_Object_Selection_Changed:
     case Sig_Object_Visible_Data_Changed:
     case Sig_Object_Property_Change_Accessible_Actions:
@@ -971,6 +957,21 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
         free_callback_para(para);
         return;
       }
+      g_object_ref(G_OBJECT(child_impl));
+      para->child_impl = child_impl;
+      break;
+    }
+    case Sig_Object_Children_Changed_Remove:
+    {
+      jobject child_ac = (*jniEnv)->GetObjectArrayElement(jniEnv, args, 1);
+      JawImpl *child_impl = jaw_impl_get_instance(jniEnv, child_ac);
+      if (child_impl == NULL)
+      {
+        JAW_DEBUG_I("child_impl == NULL");
+        free_callback_para(para);
+        return;
+      }
+      g_object_ref(G_OBJECT(child_impl));
       para->child_impl = child_impl;
       break;
     }
@@ -984,6 +985,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
         free_callback_para(para);
         return;
       }
+      g_object_ref(G_OBJECT(child_impl));
       para->child_impl = child_impl;
       break;
     }
@@ -1017,9 +1019,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_objectStateChange(JNIEnv *jniEnv
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   AtkStateType state_type = jaw_util_get_atk_state_type_from_java_state( jniEnv, state );
   para->atk_state = state_type;
   para->state_value = value;
@@ -1054,9 +1055,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentAdded(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(component_added_handler, para);
 }
 
@@ -1090,9 +1090,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentRemoved(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(component_removed_handler, para);
 }
 
@@ -1133,9 +1132,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_boundsChanged(JNIEnv *jniEnv,
     JAW_DEBUG_I("jAccContext == NULL");
     return;
   }
-  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   callback_para_process_frees();
-  CallbackPara *para = alloc_callback_para(jniEnv, global_ac);
+  CallbackPara *para = alloc_callback_para(jniEnv, jAccContext);
   jni_main_idle_add(bounds_changed_handler, para);
 }
 
