@@ -63,7 +63,8 @@ object_table_insert (JNIEnv *jniEnv, jobject ac, JawImpl* jaw_impl)
   jmethodID jmid = (*jniEnv)->GetStaticMethodID (jniEnv, atkObject, "hashCode", "(Ljavax/accessibility/AccessibleContext;)I");
   jaw_impl->hash_key = (gint)(*jniEnv)->CallStaticIntMethod (jniEnv, atkObject, jmid, ac);
   g_mutex_lock(&objectTableMutex);
-  g_hash_table_insert(objectTable, GINT_TO_POINTER(jaw_impl->hash_key), jaw_impl);
+  gboolean res = g_hash_table_insert(objectTable, GINT_TO_POINTER(jaw_impl->hash_key), jaw_impl);
+  g_assert (res == TRUE);
   g_mutex_unlock(&objectTableMutex);
 }
 
@@ -86,13 +87,20 @@ object_table_lookup (JNIEnv *jniEnv, jobject ac)
   return (JawImpl*)value;
 }
 
-static void
-object_table_remove(JNIEnv *jniEnv, JawImpl *jaw_impl)
+static gboolean
+jni_object_table_gc(gpointer data)
 {
-  JAW_DEBUG_C("%p, %p", jniEnv, jaw_impl);
-  g_mutex_lock(&objectTableMutex);
-  g_hash_table_remove(objectTable, GINT_TO_POINTER(jaw_impl->hash_key));
-  g_mutex_unlock(&objectTableMutex);
+  GSList *list = data, *cur, *next;
+
+  for (cur = list; cur != NULL; cur = next)
+  {
+    JawImpl *jaw_impl = cur->data;
+    g_object_unref(G_OBJECT(jaw_impl));
+    next = g_slist_next(cur);
+    g_slist_free_1(cur);
+  }
+
+  return G_SOURCE_REMOVE;
 }
 
 /* Called on completion of Java GC, take the opportunity to look for stale jaw_impl */
@@ -102,7 +110,7 @@ object_table_gc(JNIEnv *jniEnv)
   JAW_DEBUG_C("%p", jniEnv);
   GHashTableIter iter;
   gpointer key, value;
-  GSList *list = NULL, *cur, *next;
+  GSList *list = NULL, *cur;
 
   unsigned count[INTERFACE_MASK+1] = { 0, };
 
@@ -123,6 +131,13 @@ object_table_gc(JNIEnv *jniEnv)
 	count[jaw_impl->tflag]++;
       }
     }
+
+    /* Drop them from the hash immediately, so we don't try to GC them again */
+    for (cur = list; cur != NULL; cur = g_slist_next(cur))
+    {
+      JawImpl *jaw_impl = cur->data;
+      g_hash_table_remove(objectTable, GINT_TO_POINTER(jaw_impl->hash_key));
+    }
   }
   g_mutex_unlock(&objectTableMutex);
 
@@ -133,13 +148,8 @@ object_table_gc(JNIEnv *jniEnv)
     }
   }
 
-  for (cur = list; cur != NULL; cur = next)
-  {
-    JawImpl *jaw_impl = cur->data;
-    g_object_unref(G_OBJECT(jaw_impl));
-    next = g_slist_next(cur);
-    g_slist_free_1(cur);
-  }
+  if (list)
+    jni_main_idle_add(jni_object_table_gc, list);
 }
 
 GHashTable*
@@ -444,7 +454,7 @@ jaw_impl_get_type (guint tflag)
     typeTable = g_hash_table_new( NULL, NULL );
   }
 
-  type = GPOINTER_TO_GTYPE(g_hash_table_lookup(typeTable, GUINT_TO_POINTER(tflag)));
+  type = GPOINTER_TO_TYPE(g_hash_table_lookup(typeTable, GUINT_TO_POINTER(tflag)));
   g_mutex_unlock(&typeTableMutex);
   if (type == 0) {
     GTypeInfo tinfo = {
@@ -534,7 +544,6 @@ jaw_impl_finalize(GObject *gobject)
   JawImpl *jaw_impl = (JawImpl*)jaw_obj;
 
   JNIEnv *jniEnv = jaw_util_get_jni_env();
-  object_table_remove( jniEnv, jaw_impl );
 
   (*jniEnv)->DeleteWeakGlobalRef(jniEnv, jaw_obj->acc_context);
   jaw_obj->acc_context = NULL;
